﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// The RSS channel.
	/// </summary>
	public class RssChannel
	{
		/// <summary>
		/// The name of the channel.
		/// </summary>
		public string Title { get; set; }

		/// <summary>
		/// The URL to the HTML website corresponding to the channel.
		/// </summary>
		public Uri Link { get; set; }

		/// <summary>
		/// Phrase or sentence describing the channel.
		/// </summary>
		public string Description { get; set; }

		/// <summary>
		/// Specify one or more categories that the channel belongs to.
		/// </summary>
		public IList<RssCategory> Categories { get; internal set; }

		/// <summary>
		/// Allows processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds.
		/// </summary>
		public RssCloud Cloud { get; set; }

		/// <summary>
		/// Copyright notice for content in the channel.
		/// </summary>
		public string Copyright { get; set; }

		/// <summary>
		/// A URL that points to the documentation for the format used in the RSS file.
		/// This is defaulted to "http://www.rssboard.org/rss-specification" in the current implementation of RSS 2.0
		/// </summary>
		public string Docs { get; set; }

		/// <summary>
		/// A string indicating the program used to generate the channel.
		/// </summary>
		public string Generator { get; set; }

		/// <summary>
		/// Specifies a GIF, JPEG or PNG image that can be displayed with the channel.
		/// </summary>
		public RssImage Image { get; set; }

		/// <summary>
		/// The language the channel is written in.
		/// Values are expressed according to the two-letter ISO639 code for the language, and two-letter ISO3166 code for the country, e.g. en-US, en-GB, or it-IT
		/// </summary>
		public CultureInfo Language { get; set; }

		/// <summary>
		/// The last time the content of the channel changed.
		/// </summary>
		public DateTime? LastBuildDate { get; set; }

		/// <summary>
		/// Email address for person responsible for editorial content.
		/// </summary>
		public string ManagingEditor { get; set; }

		/// <summary>
		/// The publication date for the content in the channel.
		/// </summary>
		public DateTime? PublishedDate { get; set; }

		/// <summary>
		/// The PICS rating for the channel.
		/// </summary>
		public PicsRating Rating { get; set; }

		/// <summary>
		/// Number of minutes that indicates how long a channel can be cached before refreshing from the source.
		/// </summary>
		public int? TimeToLive { get; set; }

		/// <summary>
		/// Email address for person responsible for technical issues relating to channel.
		/// </summary>
		public string WebMaster { get; set; }

		/// <summary>
		/// A hint for aggregators telling them which days they can skip.
		/// </summary>
		public IList<DayOfWeek> SkipDays { get; internal set; }

		/// <summary>
		/// A hint for aggregators telling them which hours they can skip.
		/// </summary>
		public IList<int> SkipHours { get; internal set; }

		/// <summary>
		/// An item element represents distinct content published in the feed such as a news article, weblog entry or some other form of discrete update.
		/// A channel may contain any number of items (or no items at all).
		/// </summary>
		public IList<RssItem> Items { get; internal set; }

		/// <summary>
		/// Initializes a new instance of the <c>RssChannel</c> class with the values indicated in the input parameters.
		/// </summary>
		/// <param name="title">The title of the channel.</param>
		/// <param name="link">The URL of the web page of the channel.</param>
		/// <param name="description">The description of the channel.</param>
		/// <exception cref="System.ArgumentNullException">If title is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If link is null.</exception>
		/// <exception cref="System.ArgumentNullException">If description is null or empty.</exception>
		public RssChannel(string title, Uri link, string description)
		{
			if (String.IsNullOrEmpty(title))
			{
				throw new ArgumentNullException("title");
			}
			if (link == null)
			{
				throw new ArgumentNullException("link");
			}
			if (String.IsNullOrEmpty(description))
			{
				throw new ArgumentNullException("description");
			}

			Title = title;
			Link = link;
			Description = description;
			Docs = @"http://www.rssboard.org/rss-specification";	// default value as per RSS 2.0 specification
			Generator = "Tempesta.Web.Syndication.RssWriter";

			Items = new List<RssItem>();
			SkipDays = new List<DayOfWeek>();
			SkipHours = new List<int>();
			Categories = new List<RssCategory>();
		}

		/// <summary>
		/// Adds a category to the channel.
		/// </summary>
		/// <param name="category">Instance of the category to add.</param>
		/// <exception cref="System.ArgumentNullException">If category is null.</exception>
		public void AddCategory(RssCategory category)
		{
			if (category == null)
			{
				throw new ArgumentNullException("category");
			}

			Categories.Add(category);
		}

		/// <summary>
		/// Adds a skip day to the channel.
		/// </summary>
		/// <param name="day">Day of the week to skip.</param>
		public void AddSkipDay(DayOfWeek day)
		{
			if (!SkipDays.Contains(day))
			{
				SkipDays.Add(day);
			}
		}

		/// <summary>
		/// Adds a skip hour to the channel.
		/// </summary>
		/// <param name="hour">Hour (between 0 and 23) to skip.</param>
		/// <exception cref="System.ArgumentOutOfRangeException">If hour is less than 0 or greater than 23.</exception>
		public void AddSkipHour(int hour)
		{
			if (hour < 0 || hour > 23)
			{
				throw new ArgumentOutOfRangeException("hour", "Hour should be between 0 and 23.");
			}

			if (!SkipHours.Contains(hour))
			{
				SkipHours.Add(hour);
			}
		}

		/// <summary>
		/// Adds an item to the channel.
		/// </summary>
		/// <param name="item">Instance of the item to add.</param>
		/// <exception cref="System.ArgumentNullException">If item is null.</exception>
		public void AddItem(RssItem item)
		{
			if (item == null)
			{
				throw new ArgumentNullException("item");
			}

			Items.Add(item);
		}
	}
}
