﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// Allows processes to register with a cloud to be notified of updates to the channel, implementing a lightweight publish-subscribe protocol for RSS feeds.
	/// </summary>
	public class RssCloud
	{
		/// <summary>
		/// Host name or IP address of the web service that monitors updates to the feed.
		/// </summary>
		public string Domain { get; set; }

		/// <summary>
		/// The web service's path.
		/// </summary>
		public string Path { get; set; }

		/// <summary>
		/// The web service's TCP port.
		/// </summary>
		public int Port { get; set; }

		private string _protocol;
		/// <summary>
		/// Contains the value "xml-rpc" if the service employs XML-RPC or "soap" if it employs SOAP.
		/// </summary>
		/// <exception cref="System.ArgumentOutOfRangeException">If the value being set it not valid.</exception>
		public string Protocol
		{
			get
			{
				return _protocol;
			}
			set
			{
				if (!(new string[] { "xml-rpc", "soap" }).Contains(value))
				{
					throw new ArgumentOutOfRangeException("protocol", "Protocol must be xml-rpc or soap.");
				}

				_protocol = value;
			}
		}

		/// <summary>
		/// The remote procedure to call when requesting notification of updates.
		/// </summary>
		public string RegisterProcedure { get; set; }

		/// <summary>
		/// Initializes a new instance of the <c>RssCloud</c> class with the values indicated in the input parameters.
		/// </summary>
		/// <param name="domain">The host name or IP address of the web service.</param>
		/// <param name="path">The path of the web service.</param>
		/// <param name="port">The TCP port of the web service.</param>
		/// <param name="protocol">The protocol used by the web service. This can be either xml-rpc or soap.</param>
		/// <param name="registerProcedure">The remote procedure to call when requesting notification of updates.</param>
		/// <exception cref="System.ArgumentNullException">If domain is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If path is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If protocol is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If registerProcedure is null or empty.</exception>
		public RssCloud(string domain, string path, int port, string protocol, string registerProcedure)
		{
			if (String.IsNullOrEmpty(domain))
			{
				throw new ArgumentNullException("domain");
			}
			if (String.IsNullOrEmpty(path))
			{
				throw new ArgumentNullException("path");
			}
			if (String.IsNullOrEmpty(protocol))
			{
				throw new ArgumentNullException("protocol");
			}
			if (String.IsNullOrEmpty(registerProcedure))
			{
				throw new ArgumentNullException("registerProcedure");
			}

			Domain = domain;
			Path = path;
			Port = port;
			Protocol = protocol;
			RegisterProcedure = registerProcedure;
		}
	}

}
