﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// The image element supplies a graphical logo for the feed.
	/// </summary>
	public class RssImage
	{
		/// <summary>
		/// Max image height value, as per RSS 2.0 specification
		/// </summary>
		public const int MaxHeight = 400;

		/// <summary>
		/// Max image width value, as per RSS 2.0 specification
		/// </summary>
		public const int MaxWidth = 144;

		/// <summary>
		/// Default image height value, as per RSS 2.0 specification
		/// </summary>
		public const int DefaultHeight = 31;

		/// <summary>
		/// Default image width value, as per RSS 2.0 specification
		/// </summary>
		public const int DefaultWidth = 88;

		/// <summary>
		/// The image's title element holds character data that provides a human-readable description of the image.
		/// </summary>
		public string Title { get; set; }

		/// <summary>
		/// The image's link element identifies the URL of the web site represented by the image.
		/// </summary>
		public Uri Link { get; set; }

		private Uri _url;
		/// <summary>
		/// The image's url element identifies the URL of the image, which must be in the GIF, JPEG or PNG formats.
		/// </summary>
		/// <exception cref="System.ArgumentOutOfRangeException">If the URL does not identify a supported image format.</exception>
		public Uri Url
		{
			get
			{
				return _url;
			}
			set
			{
				if (!value.ToString().EndsWith("gif", StringComparison.InvariantCultureIgnoreCase)
					&& !value.ToString().EndsWith("jpg", StringComparison.InvariantCultureIgnoreCase)
					&& !value.ToString().EndsWith("jpeg", StringComparison.InvariantCultureIgnoreCase)
					&& !value.ToString().EndsWith("png", StringComparison.InvariantCultureIgnoreCase)
					)
				{
					throw new ArgumentOutOfRangeException("url", "Image should be in GIF, JPEG or PNG format.");
				}

				_url = value;
			}
		}

		/// <summary>
		/// The image's description element holds character data that provides a human-readable characterization of the site linked to the image.
		/// The description should be suitable for use as the title attribute of the a tag in an HTML rendering.
		/// </summary>
		public string Description { get; set; }

		private int _height;
		/// <summary>
		/// The image's height element contains the height, in pixels, of the image.
		/// The image must be no taller than 400 pixels. If this element is omitted, the image is assumed to be 31 pixels tall.
		/// </summary>
		/// <exception cref="System.ArgumentOutOfRangeException">If height is greater than the expected maximum value.</exception>
		public int Height
		{
			get
			{
				return _height;
			}
			set
			{
				if (value < 0)
				{
					throw new ArgumentOutOfRangeException("height", "Height cannot be less than zero.");
				}
				if (value > MaxHeight)
				{
					throw new ArgumentOutOfRangeException("height", String.Format("Height cannot be greater than {0}.", MaxHeight));
				}

				_height = value;
			}
		}

		private int _width;
		/// <summary>
		/// The image's width element contains the width, in pixels, of the image. The image must be no wider than 144 pixels. If this element is omitted, the image is assumed to be 88 pixels wide.
		/// </summary>
		/// <exception cref="System.ArgumentOutOfRangeException">If width is greater than the expected maximum value.</exception>
		public int Width
		{
			get
			{
				return _width;
			}
			set
			{
				if (value < 0)
				{
					throw new ArgumentOutOfRangeException("width", "Width cannot be less than zero.");
				}
				if (value > MaxWidth)
				{
					throw new ArgumentOutOfRangeException("width", String.Format("Width cannot be greater than {0}.", MaxWidth));
				}

				_width = value;
			}
		}

		/// <summary>
		/// Initializes a new instance of the <c>RssImage</c> class with the values indicated in the input parameters.
		/// </summary>
		/// <param name="title">The title of the image.</param>
		/// <param name="link">The URL of the web site represented by the image.</param>
		/// <param name="url">The URL of the image.</param>
		/// <exception cref="System.ArgumentNullException">If title is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If link is null.</exception>
		/// <exception cref="System.ArgumentNullException">If url is null.</exception>
		public RssImage(string title, Uri link, Uri url)
		{
			if (String.IsNullOrEmpty(title))
			{
				throw new ArgumentNullException("title");
			}
			if (link == null)
			{
				throw new ArgumentNullException("link");
			}
			if (url == null)
			{
				throw new ArgumentNullException("url");
			}

			Title = title;
			Link = link;
			Url = url;
			Height = DefaultHeight;
			Width = DefaultWidth;
		}
	}

}
