﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// An item element represents distinct content published in the feed such as a news article, weblog entry or some other form of discrete update.
	/// A channel may contain any number of items (or no items at all).
	/// </summary>
	public class RssItem
	{
		/// <summary>
		/// An item's guid element provides a string that uniquely identifies the item.
		/// </summary>
		public RssItem.Guid Id { get; set; }

		/// <summary>
		/// An item's title element holds character data that provides the item's headline.
		/// </summary>
		public string Title { get; set; }

		/// <summary>
		/// An item's link element identifies the URL of a web page associated with the item.
		/// </summary>
		public Uri Link { get; set; }

		/// <summary>
		/// An item's description element holds character data that contains the item's full content or a summary of its contents, a decision entirely at the discretion of the publisher.
		/// </summary>
		public string Description { get; set; }

		/// <summary>
		/// An item's pubDate element indicates the publication date and time of the item.
		/// </summary>
		public DateTime? PublishedDate { get; set; }

		/// <summary>
		/// An item's author element provides the e-mail address of the person who wrote the item.
		/// </summary>
		public string Author { get; set; }

		/// <summary>
		/// An item's category element identifies a category or tag to which the item belongs.
		/// </summary>
		public RssCategory Category { get; set; }

		/// <summary>
		/// An item's comments element identifies the URL of a web page that contains comments received in response to the item.
		/// </summary>
		public Uri Comments { get; set; }

		/// <summary>
		/// An item's enclosure element associates a media object such as an audio or video file with the item.
		/// </summary>
		public RssEnclosure Enclosure { get; set; }

		/// <summary>
		/// An item's source element indicates the fact that the item has been republished from another RSS feed.
		/// </summary>
		public RssItem.Source ItemSource { get; set; }

		/// <summary>
		/// Initializes a new instance of the <c>RssItem</c> class with the value indicated in the input parameter and an empty Id.
		/// </summary>
		/// <param name="title">The title of the item.</param>
		/// <exception cref="System.ArgumentNullException">If title is null or empty.</exception>
		public RssItem(string title)
		{
			if (String.IsNullOrEmpty(title))
			{
				throw new ArgumentNullException("title");
			}

			Title = title;
		}

		/// <summary>
		/// Initializes a new instance of the <c>RssItem</c> class with the values indicated in the input parameters.
		/// </summary>
		/// <param name="title">The title of the item.</param>
		/// <param name="id">The unique identifier of the item.</param>
		/// <exception cref="System.ArgumentNullException">If title is null or empty.</exception>
		/// <exception cref="System.ArgumentNullException">If id is null.</exception>
		public RssItem(string title, Guid id)
		{
			if (String.IsNullOrEmpty(title))
			{
				throw new ArgumentNullException("title");
			}
			if (id == null)
			{
				throw new ArgumentNullException("id");
			}

			Title = title;
			Id = id;
		}

		/// <summary>
		/// An item's guid element provides a string that uniquely identifies the item.
		/// The guid may include an isPermaLink attribute.
		/// The guid enables an aggregator to detect when an item has been received previously and does not need to be presented to a user again.
		/// If the guid's isPermaLink attribute is omitted or has the value "true", the guid must be the permanent URL of the web page associated with the item.
		/// If the guid's isPermaLink attribute has the value "false", the guid may employ any syntax the feed's publisher has devised for ensuring the uniqueness of the string, such as the Tag URI scheme described in RFC 4151.
		/// </summary>
		public class Guid
		{
			private Uri _permaLink;
			/// <summary>
			/// The permanent URL of the web page associated with the item.
			/// When setting this property, the IsPermaLink flag is set to true.
			/// </summary>
			public Uri PermaLink
			{
				get
				{
					return _permaLink;
				}
				set
				{
					_permaLink = value;
					IsPermaLink = true;
				}
			}

			private string _tag;
			/// <summary>
			/// The custom unique identifier of the item, to be used as an alternative to the PermaLink.
			/// When setting this property, the IsPermaLink flag is set to false.
			/// </summary>
			public string Tag
			{
				get
				{
					return _tag;
				}
				set
				{
					_tag = value;
					IsPermaLink = false;
				}
			}

			/// <summary>
			/// Read-only value: if true, a PermaLink is used; if false, a custom Tag is used as unique identifier of the item.
			/// </summary>
			public bool IsPermaLink { get; private set; }

			/// <summary>
			/// Initializes a new instance of the <c>RssItem</c>.<c>Guid</c> class as a perma link.
			/// The IsPermaLink property is automatically set to true.
			/// </summary>
			/// <param name="permaLink">The perma link of the RSS item.</param>
			/// <exception cref="System.ArgumentNullException">If permaLink is null.</exception>
			public Guid(Uri permaLink)
			{
				if (permaLink == null)
				{
					throw new ArgumentNullException("permaLink");
				}

				PermaLink = permaLink;
			}

			/// <summary>
			/// Initializes a new instance of the <c>RssItem</c>.<c>Guid</c> class as with the custom identifier provided.
			/// The IsPermaLink property is automatically set to false.
			/// </summary>
			/// <param name="tag">The custom identifier of the RSS item.</param>
			/// <exception cref="System.ArgumentNullException">If tag is null or empty.</exception>
			public Guid(string tag)
			{
				if (String.IsNullOrEmpty(tag))
				{
					throw new ArgumentNullException("tag");
				}

				Tag = tag;
			}
		}

		/// <summary>
		/// An item's source element indicates the fact that the item has been republished from another RSS feed.
		/// </summary>
		public class Source
		{
			/// <summary>
			/// Title of the source identified at the provided URL.
			/// </summary>
			public string Title { get; set; }

			/// <summary>
			/// Identifies the URL of the source feed.
			/// </summary>
			public Uri Url { get; set; }

			/// <summary>
			///  Initializes a new instance of the <c>RssItem</c>.<c>Source</c> class with the values indicated in the input parameters.
			/// </summary>
			/// <param name="title">The title of the source of the RSS item.</param>
			/// <param name="url">The url of the source of the RSS item.</param>
			/// <exception cref="System.ArgumentNullException">If title is null or empty.</exception>
			/// <exception cref="System.ArgumentNullException">If url is null.</exception>
			public Source(string title, Uri url)
			{
				if (String.IsNullOrEmpty(title))
				{
					throw new ArgumentNullException("title");
				}
				if (url == null)
				{
					throw new ArgumentNullException("url");
				}

				Title = title;
				Url = url;
			}
		}
	}
}
