﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Globalization;
using System.Net;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// Implementation of a RSS 2.0 reader.
	/// </summary>
	public class RssReader : IDisposable
	{
		private XmlReader _reader = null;

		/// <summary>
		/// List of supported RSS versions.
		/// </summary>
		public readonly string[] SupportedVersions = new string[] { "2.0" };

		/// <summary>
		/// Initializes a new instance of the <c>RssReader</c> class with default properties.
		/// </summary>
		public RssReader()
		{
		}

		/// <summary>
		/// Reads the RSS XML from the input URI.
		/// </summary>
		/// <param name="input">Instance of the input URI.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If input is null.</exception>
		public RssFeed Read(Uri input)
		{
			if (input == null)
			{
				throw new ArgumentNullException("input");
			}

			Stream rssStream = null;
			using (WebClient webClient = new WebClient())
			{
				rssStream = webClient.OpenRead(input);
			}

			return Read(rssStream);
		}

		/// <summary>
		/// Reads the RSS XML from the input stream.
		/// </summary>
		/// <param name="input">Instance of the input stream.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If input is null.</exception>
		public RssFeed Read(Stream input)
		{
			if (input == null)
			{
				throw new ArgumentNullException("input");
			}

			_reader = XmlReader.Create(input, new XmlReaderSettings() { ConformanceLevel = ConformanceLevel.Document });

			return ReadXml();
		}

		/// <summary>
		/// Reads the RSS XML from a file identified by the provided filename, including full access path.
		/// </summary>
		/// <param name="inputFilename">Input file name, including full access path.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If inputFilename is null or empty.</exception>
		public RssFeed Read(string inputFilename)
		{
			if (String.IsNullOrEmpty(inputFilename))
			{
				throw new ArgumentNullException("inputFilename");
			}

			_reader = XmlReader.Create(inputFilename, new XmlReaderSettings() { ConformanceLevel = ConformanceLevel.Document });

			return ReadXml();
		}

		/// <summary>
		/// Reads the RSS XML from the input string builder.
		/// </summary>
		/// <param name="input">Instance of the input string builder.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If input is null.</exception>
		public RssFeed Read(StringBuilder input)
		{
			if (input == null)
			{
				throw new ArgumentNullException("input");
			}

			return Read(new StringReader(input.ToString()));
		}

		/// <summary>
		/// Reads the RSS XML from the input text writer.
		/// </summary>
		/// <param name="input">Instance of the input text writer.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If input is null.</exception>
		public RssFeed Read(TextReader input)
		{
			if (input == null)
			{
				throw new ArgumentNullException("input");
			}

			_reader = XmlReader.Create(input, new XmlReaderSettings() { ConformanceLevel = ConformanceLevel.Document });

			return ReadXml();
		}

		/// <summary>
		/// Reads the RSS XML from the input XML writer.
		/// </summary>
		/// <param name="input">Instance of the input XML writer.</param>
		/// <returns>An instance of the <c>RssFeed</c> class that contains the RSS channel and items.</returns>
		/// <exception cref="System.ArgumentNullException">If input is null.</exception>
		public RssFeed Read(XmlReader input)
		{
			if (input == null)
			{
				throw new ArgumentNullException("input");
			}

			_reader = XmlReader.Create(input, new XmlReaderSettings() { ConformanceLevel = ConformanceLevel.Document });

			return ReadXml();
		}

		#region Implement IDisposable
		/// <summary>
		/// Destroys the instance of the <c>RssReader</c> class. Disposal of managed resources owned by this object is also performed.
		/// </summary>
		~RssReader()
		{
			Dispose(false);
		}

		/// <summary>
		/// Disposes managed resources owned by this object.
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		/// <summary>
		/// Implementation of the <c>IDisposable</c> interface.
		/// </summary>
		/// <param name="disposing">Flag to determine whether managed resources owned by the object should be disposed (true) or not (false).</param>
		protected virtual void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (_reader != null)
				{
					_reader.Close();
					_reader = null;
				}
			}
		}
		#endregion

		//public RssFeed Read<TKey>(int top, Func<RssItem, TKey> orderBy)
		//{
		//    RssFeed rss = new RssFeed();
		//    var items = rss.Channel.Items.OrderBy(orderBy).Take(top);
		//}

		/// <summary>
		/// Reads the XML representation of the RSS feed using an <c>XmlReader</c> object internally.
		/// </summary>
		protected RssFeed ReadXml()
		{
			_reader.MoveToContent();

			ReadRss();

			RssFeed rss = new RssFeed();
			ReadChannel(rss);

			_reader.Close();

			return rss;
		}

		/// <summary>
		/// Reads the root &lt;rss&gt; tag of the RSS feed.
		/// </summary>
		/// <exception cref="System.InvalidOperationException">If the rss node does not exist.</exception>
		/// <exception cref="System.InvalidOperationException">If the version attribute in the rss node is not specified.</exception>
		/// <exception cref="System.InvalidOperationException">If the version specified is not supported.</exception>
		protected void ReadRss()
		{
			if (_reader.Name != "rss")
			{
				throw new InvalidOperationException("Required rss node not found.");
			}

			// read version attribute of the root <rss> node
			string rssVersion = _reader.GetAttribute("version");

			if (String.IsNullOrEmpty(rssVersion))
			{
				throw new InvalidOperationException("Required version attribute not found for rss node.");
			}

			if (!SupportedVersions.Contains(rssVersion))
			{
				throw new InvalidOperationException(String.Format("Version {0} is not supported.", rssVersion));
			}
		}

		/// <summary>
		/// Reads the &lt;channel&gt; tag of the RSS feed.
		/// </summary>
		/// <exception cref="System.InvalidOperationException">If the channel node does not exist.</exception>
		/// <exception cref="System.InvalidOperationException">If the title node in the channel does not exist.</exception>
		/// <exception cref="System.InvalidOperationException">If the link node in the channel does not exist.</exception>
		/// <exception cref="System.InvalidOperationException">If the description node in the channel does not exist.</exception>
		protected void ReadChannel(RssFeed rss)
		{
			if (!_reader.ReadToFollowing("channel"))
			{
				throw new InvalidOperationException("Required channel node not found.");
			}

			Dictionary<string, object> channel = new Dictionary<string, object>();
			IList<RssCategory> categories = new List<RssCategory>();
			IList<RssItem> items = new List<RssItem>();

			do
			{
				if (_reader.NodeType == XmlNodeType.Element)
				{
					switch (_reader.Name)
					{
						case "title":
							channel["title"] = ReadTitle();
							break;

						case "link":
							channel["link"] = ReadLink();
							break;

						case "description":
							channel["description"] = ReadDescription();
							break;

						case "category":
							RssCategory category = ReadCategory();
							if (category != null)
							{
								categories.Add(category);
							}
							break;

						case "cloud":
							channel["cloud"] = ReadCloud();
							break;

						case "copyright":
							channel["copyright"] = ReadCopyright();
							break;

						case "docs":
							channel["docs"] = ReadDocs();
							break;

						case "generator":
							channel["generator"] = ReadGenerator();
							break;

						case "image":
							channel["image"] = ReadImage();
							break;

						case "language":
							channel["language"] = ReadLanguage();
							break;

						case "lastBuildDate":
							channel["lastBuildDate"] = ReadLastBuildDate();
							break;

						case "managingEditor":
							channel["managingEditor"] = ReadManagingEditor();
							break;

						case "pubDate":
							channel["pubDate"] = ReadPubDate();
							break;

						case "rating":
							channel["rating"] = ReadRating();
							break;

						case "ttl":
							channel["ttl"] = ReadTtl();
							break;

						case "webMaster":
							channel["webMaster"] = ReadWebMaster();
							break;

						case "skipDays":
							channel["skipDays"] = ReadSkipDays();
							break;

						case "skipHours":
							channel["skipHours"] = ReadSkipHours();
							break;

						case "item":
							RssItem item = ReadItem();
							if (item != null)
							{
								items.Add(item);
							}
							break;

						default:
							break;
					}
				}
			} while (_reader.Read());

			if (!channel.ContainsKey("title"))
			{
				throw new InvalidOperationException("Required title node not found in the channel.");
			}

			if (!channel.ContainsKey("link"))
			{
				throw new InvalidOperationException("Required link node not found in the channel.");
			}

			if (!channel.ContainsKey("description"))
			{
				throw new InvalidOperationException("Required description node not found in the channel.");
			}
			
			rss.Channel = new RssChannel(channel["title"] as string, channel["link"] as Uri, channel["description"] as string)
				{
					Categories = categories,
					Cloud = channel.ContainsKey("cloud") ? channel["cloud"] as RssCloud : null,
					Copyright = channel.ContainsKey("copyright") ? channel["copyright"] as string : null,
					Docs = channel.ContainsKey("docs") ? channel["docs"] as string : null,
					Generator = channel.ContainsKey("generator") ? channel["generator"] as string : null,
					Image = channel.ContainsKey("image") ? channel["image"] as RssImage : null,
					Language = channel.ContainsKey("language") ? channel["language"] as CultureInfo : null,
					LastBuildDate = channel.ContainsKey("lastBuildDate") ? channel["lastBuildDate"] as DateTime? : null,
					ManagingEditor = channel.ContainsKey("managingEditor") ? channel["managingEditor"] as string : null,
					PublishedDate = channel.ContainsKey("pubDate") ? channel["pubDate"] as DateTime? : null,
					Rating = channel.ContainsKey("rating") ? channel["rating"] as PicsRating : null,
					TimeToLive = channel.ContainsKey("ttl") ? channel["ttl"] as int? : null,
					WebMaster = channel.ContainsKey("webMaster") ? channel["webMaster"] as string : null,
					SkipDays = channel.ContainsKey("skipDays") ? channel["skipDays"] as IList<DayOfWeek> : new List<DayOfWeek>(),
					SkipHours = channel.ContainsKey("skipHours") ? channel["skipHours"] as IList<int> : new List<int>(),
					Items = items
				};
		}

		/// <summary>
		/// Reads the &lt;title&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the title node.</returns>
		protected string ReadTitle()
		{
			// <title>Liftoff News</title>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;link&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>Uri</c> class containing the content of the link node.</returns>
		protected Uri ReadLink()
		{
			// <link>http://liftoff.msfc.nasa.gov/</link>
			string strVal = _reader.ReadString();
			Uri uriVal;

			if (Uri.TryCreate(strVal, UriKind.RelativeOrAbsolute, out uriVal))
			{
				return uriVal;
			}

			return null;
		}

		/// <summary>
		/// Reads the &lt;description&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the description node.</returns>
		protected string ReadDescription()
		{
			// <description>Liftoff to Space Exploration.</description>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;category&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>RssCategory</c> class containing the content of the category node.</returns>
		protected RssCategory ReadCategory()
		{
			// <category domain="dmoz">News/Newspapers/Regional/United_States/Texas</category>
			string domain = _reader.GetAttribute("domain");
			string name = _reader.ReadString();

			if (String.IsNullOrEmpty(name))
			{
				return null;
			}

			if (String.IsNullOrEmpty(domain))
			{
				return new RssCategory(name);
			}

			return new RssCategory(name, domain);
		}

		/// <summary>
		/// Reads the &lt;cloud&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>RssCloud</c> class containing the content of the cloud node.</returns>
		protected RssCloud ReadCloud()
		{
			// <cloud domain="server.example.com" path="/rpc" port="80" protocol="xml-rpc" registerProcedure="cloud.notify" />
			string domain = _reader.GetAttribute("domain");
			string path = _reader.GetAttribute("path");
			string protocol = _reader.GetAttribute("protocol");
			string registerProcedure = _reader.GetAttribute("registerProcedure");
			string portVal = _reader.GetAttribute("port");
			
			int port;
			if (String.IsNullOrEmpty(domain) 
				|| String.IsNullOrEmpty(path) 
				|| String.IsNullOrEmpty(protocol) 
				|| String.IsNullOrEmpty(registerProcedure)
				|| !int.TryParse(portVal, out port)
				)
			{
				return null;
			}

			return new RssCloud(domain, path, port, protocol, registerProcedure);
		}

		/// <summary>
		/// Reads the &lt;copyright&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the copyright node.</returns>
		protected string ReadCopyright()
		{
			// <copyright>Copyright 2007 Dallas Times-Herald</copyright>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;docs&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the docs node.</returns>
		protected string ReadDocs()
		{
			// <docs>http://www.rssboard.org/rss-specification</docs>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;generator&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the generator node.</returns>
		protected string ReadGenerator()
		{
			// <generator>Microsoft Spaces v1.1</generator>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;image&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>RssImage</c> class containing the content of the image node.</returns>
		protected RssImage ReadImage()
		{
			// <image>
			//	 <link>http://dallas.example.com</link>
			//	 <title>Dallas Times-Herald</title>
			//	 <url>http://dallas.example.com/masthead.gif</url>
			//	 <description>Read the Dallas Times-Herald</description>
			//	 <height>32</height>
			//	 <width>96</width>
			// </image>
			Uri link = null;
			string title = null;
			Uri url = null;
			string description = null;
			int? height = default(int?);
			int? width = default(int?);

			bool endImage = false;
			bool endFile = false;

			while (!endImage && !endFile)
			{
				if (_reader.NodeType == XmlNodeType.Element)
				{
					switch (_reader.Name)
					{
						case "link":
							string linkVal = _reader.ReadString();
							Uri.TryCreate(linkVal, UriKind.RelativeOrAbsolute, out link);
							break;

						case "title":
							title = _reader.ReadString();
							break;

						case "url":
							string urlVal = _reader.ReadString();
							Uri.TryCreate(urlVal, UriKind.RelativeOrAbsolute, out url);
							break;

						case "description":
							description = _reader.ReadString();
							break;

						case "height":
							string heightVal = _reader.ReadString();
							int heightOut;
							if (int.TryParse(heightVal, out heightOut))
							{
								height = heightOut;
							}
							break;

						case "width":
							string widthVal = _reader.ReadString();
							int widthOut;
							if (int.TryParse(widthVal, out widthOut))
							{
								width = widthOut;
							}
							break;

						default:
							break;
					}
				}

				endFile = !_reader.Read();
				endImage = _reader.NodeType == XmlNodeType.EndElement && _reader.Name == "image";
			};

			if (String.IsNullOrEmpty(title)
				|| link == null
				|| url == null
				)
			{
				return null;
			}

			RssImage image = new RssImage(title, link, url);

			if (!String.IsNullOrEmpty(description))
			{
				image.Description = description;
			}

			if (height.HasValue)
			{
				image.Height = height.Value;
			}

			if (width.HasValue)
			{
				image.Width = width.Value;
			}

			return image;
		}

		/// <summary>
		/// Reads the &lt;language&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>CultureInfo</c> class containing the language defined in the content of the language node.</returns>
		protected CultureInfo ReadLanguage()
		{
			// <language>en-US</language>
			try
			{
				return CultureInfo.CreateSpecificCulture(_reader.ReadString());
			}
			catch
			{
				return null;
			}
		}

		/// <summary>
		/// Reads the &lt;lastBuildDate&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An nullable instance of <c>DateTime</c> containing the date defined in the content of the lastBuildDate node.</returns>
		protected DateTime? ReadLastBuildDate()
		{
			// <lastBuildDate>Sun, 14 Oct 2007 17:17:44 GMT</lastBuildDate>
			DateTime lastBuildDate;
			bool validDate = DateTime.TryParse(_reader.ReadString(), out lastBuildDate);

			return validDate ? new Nullable<DateTime>(lastBuildDate) : null;
		}

		/// <summary>
		/// Reads the &lt;managingEditor&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the managingEditor node.</returns>
		protected string ReadManagingEditor()
		{
			// <managingEditor>jlehrer@dallas.example.com (Jim Lehrer)</managingEditor>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;pubDate&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An nullable instance of <c>DateTime</c> containing date defined in the content of the pubDate node.</returns>
		protected DateTime? ReadPubDate()
		{
			// <pubDate>Sun, 14 Oct 2007 05:00:00 GMT</pubDate>
			DateTime pubDate;
			bool validDate = DateTime.TryParse(_reader.ReadString(), out pubDate);

			return validDate ? new Nullable<DateTime>(pubDate) : null;
		}

		/// <summary>
		/// Reads the &lt;rating&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>PicsRating</c> class containing the content of the rating node.</returns>
		protected PicsRating ReadRating()
		{
			// <rating>(PICS-1.1 "http://www.rsac.org/ratingsv01.html" l by "webmaster@example.com" on "2007.01.29T10:09-0800" r (n 0 s 0 v 0 l 0))</rating>
			string ratingVal = _reader.ReadString();

			return String.IsNullOrEmpty(ratingVal) ? null : new PicsRating(ratingVal);
		}

		/// <summary>
		/// Reads the &lt;ttl&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An nullable unsigned integer number containing the content of the ttl node.</returns>
		protected int? ReadTtl()
		{
			// <ttl>60</ttl>
			string ttlVal = _reader.ReadString();
			int ttlOut;
			
			return int.TryParse(ttlVal, out ttlOut) ? new Nullable<int>(ttlOut) : null;
		}

		/// <summary>
		/// Reads the &lt;webMaster&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>The content of the webMaster node.</returns>
		protected string ReadWebMaster()
		{
			// <webMaster>helpdesk@dallas.example.com</webMaster>
			return _reader.ReadString();
		}

		/// <summary>
		/// Reads the &lt;skipDays&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>A list of the <c>DayOfWeek</c> enum containing the days defined in the content of the skipDays node.</returns>
		protected IList<DayOfWeek> ReadSkipDays()
		{
			// <skipDays>
			//	 <day>Saturday</day>
			//	 <day>Sunday</day>
			// </skipDays>
			List<DayOfWeek> days = new List<DayOfWeek>();

			bool endFile = false;
			bool endSkipDays = false;

			while (!endSkipDays && !endFile)
			{
				if (_reader.NodeType == XmlNodeType.Element && _reader.Name == "day")
				{
					DayOfWeek day;
					if (Enum.TryParse<DayOfWeek>(_reader.ReadString(), out day))
					{
						days.Add(day);
					}
				}

				endFile = !_reader.Read();
				endSkipDays = _reader.NodeType == XmlNodeType.EndElement && _reader.Name == "skipDays";
			};

			return days.Count > 0 ? days : null;
		}

		/// <summary>
		/// Reads the &lt;skipHours&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>A list of unsigned integer numbers containing the hours defined in the content of the skipHours node.</returns>
		protected IList<uint> ReadSkipHours()
		{
			// <skipHours>
			//	 <hour>0</hour>
			//	 <hour>1</hour>
			//	 <hour>2</hour>
			//	 <hour>22</hour>
			//	 <hour>23</hour>
			//	</skipHours>
			List<uint> hours = new List<uint>();

			bool endFile = false;
			bool endSkipHours = false;

			while (!endSkipHours && !endFile)
			{
				if (_reader.NodeType == XmlNodeType.Element && _reader.Name == "hour")
				{
					uint hour;
					if (uint.TryParse(_reader.ReadString(), out hour))
					{
						hours.Add(hour);
					}
				}

				endFile = !_reader.Read();
				endSkipHours = _reader.NodeType == XmlNodeType.EndElement && _reader.Name == "skipHours";
			};
			
			return hours.Count > 0 ? hours : null;
		}

		/// <summary>
		/// Reads the &lt;item&gt; node of the channel in the RSS feed.
		/// </summary>
		/// <returns>An instance of the <c>RssItem</c> class containing the content of the item node.</returns>
		protected RssItem ReadItem()
		{
			// <item>
			//	 <guid isPermaLink="false">tag:dallas.example.com,4131:news</guid>
			//   <title>Seventh Heaven! Ryan Hurls Another No Hitter</title>
			//	 <author>jbb@dallas.example.com (Joe Bob Briggs)</author>
			//	 <category domain="rec.arts.movies.reviews">1983/V</category>
			//	 <comments>http://dallas.example.com/feedback/1983/06/joebob.htm</comments>
			//	 <description><![CDATA[Texas Rangers pitcher Nolan Ryan hurled]]></description>
			//	 <enclosure length="24986239" type="audio/mpeg" url="http://dallas.example.com/joebob_050689.mp3" />
			//	 <link>http://dallas.example.com/1991/05/02/nolan.htm</link>
			//	 <pubDate>Fri, 05 Oct 2007 09:00:00 CST</pubDate>
			//	 <source url="http://la.example.com/rss.xml">Los Angeles Herald-Examiner</source>
			// </item>
			RssItem.Guid guid = null;
			string title = null;
			string author = null;
			RssCategory category = null;
			Uri comments = null;
			string description = null;
			RssEnclosure enclosure = null;
			Uri link = null;
			DateTime? pubDate = null;
			RssItem.Source source = null;

			bool endItem = false;
			bool endFile = false;

			while (!endItem && !endFile)
			{
				if (_reader.NodeType == XmlNodeType.Element)
				{
					switch (_reader.Name)
					{
						case "guid":
							string isPermaLink = _reader.GetAttribute("isPermaLink");

							if (String.IsNullOrEmpty(isPermaLink) || isPermaLink.Equals("true"))
							{
								string guidVal = _reader.ReadString();
								Uri guidOut;
								if (Uri.TryCreate(guidVal, UriKind.RelativeOrAbsolute, out guidOut))
								{
									guid = new RssItem.Guid(guidOut);
								}
							}
							else
							{
								guid = new RssItem.Guid(_reader.ReadString());
							}
							break;

						case "title":
							title = _reader.ReadString();
							break;

						case "author":
							author = _reader.ReadString();
							break;

						case "category":
							category = ReadCategory();
							break;

						case "comments":
							string commentsVal = _reader.ReadString();
							Uri.TryCreate(commentsVal, UriKind.RelativeOrAbsolute, out comments);
							break;

						case "description":
							// may or may not have a CDATA section
							description = _reader.ReadString();
							break;

						case "enclosure":
							string type = _reader.GetAttribute("type");
							string urlVal = _reader.GetAttribute("url");
							string lengthVal = _reader.GetAttribute("length");
							
							Uri urlOut;
							long lengthOut;
							if (!String.IsNullOrEmpty(type)
								&& Uri.TryCreate(urlVal, UriKind.RelativeOrAbsolute, out urlOut)
								&& long.TryParse(lengthVal, out lengthOut)
								)
							{
								enclosure = new RssEnclosure(type, urlOut, lengthOut);
							}
							break;

						case "link":
							string linkVal = _reader.ReadString();
							Uri.TryCreate(linkVal, UriKind.RelativeOrAbsolute, out link);
							break;

						case "pubDate":
							DateTime dateOut;
							pubDate = DateTime.TryParse(_reader.ReadString(), out dateOut) ? new Nullable<DateTime>(dateOut) : null;
							break;

						case "source":
							string sourceName = _reader.ReadString();
							string sourceUrl = _reader.GetAttribute("url");
							Uri sourceUrlOut;

							if (!String.IsNullOrEmpty(sourceName)
								&& Uri.TryCreate(sourceUrl, UriKind.RelativeOrAbsolute, out sourceUrlOut)
								)
							{
								source = new RssItem.Source(sourceName, sourceUrlOut);
							}
							break;

						default:
							break;
					}
				}

				endFile = !_reader.Read();
				endItem = _reader.NodeType == XmlNodeType.EndElement && _reader.Name == "item";
			};

			// either title or description must exist for a valid item
			// if title doesn't exist, description is used
			title = title ?? description;

			if (String.IsNullOrEmpty(title))
			{
				return null;
			}

			return new RssItem(title)
				{
					Author = author,
					Category = category,
					Comments = comments,
					Description = description,
					Enclosure = enclosure,
					Id = guid,
					ItemSource = source,
					Link = link,
					PublishedDate = pubDate
				};
		}
	}
}
