﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;

namespace Tempesta.Web.Syndication
{
	/// <summary>
	/// Implementation of a RSS 2.0 writer.
	/// </summary>
	public class RssWriter : IDisposable
	{
		private XmlWriter _writer = null;
		private RssFeed _rss = null;

		/// <summary>
		/// Initializes a new instance of the RssWriter class with the <c>RssFeed</c> object specified in input.
		/// </summary>
		public RssWriter(RssFeed rss)
		{
			if (rss == null)
			{
				throw new ArgumentNullException("rss");
			}

			_rss = rss;
		}

		/// <summary>
		/// Spools the RSS XML to the output stream.
		/// </summary>
		/// <param name="output">Instance of the output stream.</param>
		/// <exception cref="System.ArgumentNullException">If output is null.</exception>
		public void Write(Stream output)
		{
			if (output == null)
			{
				throw new ArgumentNullException("output");
			}

			_writer = XmlWriter.Create(output);
			CreateXml();
		}

		/// <summary>
		/// Spools the RSS XML to a file identified by the provided filename, including full access path.
		/// </summary>
		/// <param name="outputFileName">Output file name, including full access path.</param>
		/// <exception cref="System.ArgumentNullException">If outputFilename is null or empty.</exception>
		public void Write(string outputFileName)
		{
			if (String.IsNullOrEmpty(outputFileName))
			{
				throw new ArgumentNullException("output");
			}

			_writer = XmlWriter.Create(outputFileName);
			CreateXml();
		}

		/// <summary>
		/// Spools the RSS XML to the output string builder.
		/// </summary>
		/// <param name="output">Instance of the output string builder.</param>
		/// <exception cref="System.ArgumentNullException">If output is null.</exception>
		public void Write(StringBuilder output)
		{
			if (output == null)
			{
				throw new ArgumentNullException("output");
			}

			_writer = XmlWriter.Create(output);
			CreateXml();
		}

		/// <summary>
		/// Spools the RSS XML to the output text writer.
		/// </summary>
		/// <param name="output">Instance of the output text writer.</param>
		/// <exception cref="System.ArgumentNullException">If output is null.</exception>
		public void Write(TextWriter output)
		{
			if (output == null)
			{
				throw new ArgumentNullException("output");
			}

			_writer = XmlWriter.Create(output);
			CreateXml();
		}

		/// <summary>
		/// Spools the RSS XML to the output XML writer.
		/// </summary>
		/// <param name="output">Instance of the output XML writer.</param>
		/// <exception cref="System.ArgumentNullException">If output is null.</exception>
		public void Write(XmlWriter output)
		{
			if (output == null)
			{
				throw new ArgumentNullException("output");
			}

			_writer = XmlWriter.Create(output);
			CreateXml();
		}

		#region Implement IDisposable
		/// <summary>
		/// Destroys the instance of the RssWriter class. Disposal of managed resources owned by this object is also performed.
		/// </summary>
		~RssWriter()
		{
			Dispose(false);
		}

		/// <summary>
		/// Disposes managed resources owned by this object.
		/// </summary>
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		/// <summary>
		/// Implementation of the IDisposable interface.
		/// </summary>
		/// <param name="disposing">Flag to determine whether managed resources owned by the object should be disposed (true) or not (false).</param>
		protected virtual void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (_writer != null)
				{
					_writer.Close();
					_writer = null;
				}
			}
		}
		#endregion

		/// <summary>
		/// Generates the XML representation of the RSS feed using an XmlWriter object internally.
		/// </summary>
		protected void CreateXml()
		{
			if (_writer == null) return;

			_writer.WriteStartDocument();
			_writer.WriteStartElement("rss");
			_writer.WriteAttributeString("version", "2.0");

			_writer.WriteStartElement("channel");

			WriteTitle();
			WriteLink();
			WriteDescription();
			WriteCategories();
			WriteCloud();
			WriteDocs();
			WriteGenerator();
			WriteImage();
			WriteLanguage();
			WriteLastBuildDate();
			WriteManagingEditor();
			WritePubDate();
			WriteRating();
			WriteSkipDays();
			WriteSkipHours();
			WriteTtl();
			WriteWebMaster();
			WriteItems();

			_writer.WriteEndElement();
			_writer.WriteEndDocument();

			_writer.Close();
		}

		/// <summary>
		/// Writes the &lt;title&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteTitle()
		{
			// <title>Liftoff News</title>
			_writer.WriteElementString("title", _rss.Channel.Title);
		}

		/// <summary>
		/// Writes the &lt;link&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteLink()
		{
			// <link>http://liftoff.msfc.nasa.gov/</link>
			_writer.WriteElementString("link", _rss.Channel.Link.ToString());
		}

		/// <summary>
		/// Writes the &lt;description&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteDescription()
		{
			// <description>Liftoff to Space Exploration.</description>
			_writer.WriteElementString("description", _rss.Channel.Description);
		}

		/// <summary>
		/// Writes one or more &lt;category&gt; nodes of the channel in the RSS feed.
		/// </summary>
		protected void WriteCategories()
		{
			// <category domain="dmoz">News/Newspapers/Regional/United_States/Texas</category>
			foreach (var category in _rss.Channel.Categories)
			{
				_writer.WriteStartElement("category");

				if (!String.IsNullOrEmpty(category.Domain))
				{
					_writer.WriteAttributeString("domain", category.Domain);
				}

				_writer.WriteString(category.Name);

				_writer.WriteEndElement();
			}
		}

		/// <summary>
		/// Writes the &lt;cloud&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteCloud()
		{
			// <cloud domain="server.example.com" path="/rpc" port="80" protocol="xml-rpc" registerProcedure="cloud.notify" />
			if (_rss.Channel.Cloud != null)
			{
				_writer.WriteStartElement("cloud");

				_writer.WriteAttributeString("domain", _rss.Channel.Cloud.Domain);
				_writer.WriteAttributeString("path", _rss.Channel.Cloud.Path);
				_writer.WriteAttributeString("port", _rss.Channel.Cloud.Port.ToString());
				_writer.WriteAttributeString("protocol", _rss.Channel.Cloud.Protocol);
				_writer.WriteAttributeString("registerProcedure", _rss.Channel.Cloud.RegisterProcedure);

				_writer.WriteEndElement();
			}
		}

		/// <summary>
		/// Writes the &lt;copyright&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteCopyright()
		{
			// <copyright>Copyright 2007 Dallas Times-Herald</copyright>
			if (String.IsNullOrEmpty(_rss.Channel.Copyright))
			{
				_writer.WriteElementString("copyright", _rss.Channel.Copyright);
			}
		}

		/// <summary>
		/// Writes the &lt;docs&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteDocs()
		{
			// <docs>http://www.rssboard.org/rss-specification</docs>
			if (String.IsNullOrEmpty(_rss.Channel.Docs))
			{
				_writer.WriteElementString("docs", _rss.Channel.Docs);
			}
		}

		/// <summary>
		/// Writes the &lt;generator&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteGenerator()
		{
			// <generator>Microsoft Spaces v1.1</generator>
			if (String.IsNullOrEmpty(_rss.Channel.Generator))
			{
				_writer.WriteElementString("generator", _rss.Channel.Generator);
			}
		}

		/// <summary>
		/// Writes the &lt;image&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteImage()
		{
			// <image>
			//	 <link>http://dallas.example.com</link>
			//	 <title>Dallas Times-Herald</title>
			//	 <url>http://dallas.example.com/masthead.gif</url>
			//	 <description>Read the Dallas Times-Herald</description>
			//	 <height>32</height>
			//	 <width>96</width>
			// </image>
			if (_rss.Channel.Image != null)
			{
				_writer.WriteStartElement("image");

				_writer.WriteElementString("link", _rss.Channel.Image.Link.ToString());
				_writer.WriteElementString("title", _rss.Channel.Image.Title);
				_writer.WriteElementString("url", _rss.Channel.Image.Url.ToString());

				if (!String.IsNullOrEmpty(_rss.Channel.Image.Description))
				{
					_writer.WriteElementString("title", _rss.Channel.Image.Description);
				}

				_writer.WriteElementString("height", _rss.Channel.Image.Height.ToString());
				_writer.WriteElementString("width", _rss.Channel.Image.Width.ToString());

				_writer.WriteEndElement();
			}
		}

		/// <summary>
		/// Writes the &lt;language&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteLanguage()
		{
			// <language>en-US</language>
			if (_rss.Channel.Language != null)
			{
				_writer.WriteElementString("language", _rss.Channel.Language.Name);
			}
		}

		/// <summary>
		/// Writes the &lt;lastBuildDate&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteLastBuildDate()
		{
			// <lastBuildDate>Sun, 14 Oct 2007 17:17:44 GMT</lastBuildDate>
			if (_rss.Channel.LastBuildDate != null)
			{
				_writer.WriteElementString("lastBuildDate", _rss.Channel.LastBuildDate.Value.ToLongDateString());
			}
		}

		/// <summary>
		/// Writes the &lt;managingEditor&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteManagingEditor()
		{
			// <managingEditor>jlehrer@dallas.example.com (Jim Lehrer)</managingEditor>
			if (!String.IsNullOrEmpty(_rss.Channel.ManagingEditor))
			{
				_writer.WriteElementString("managingEditor", _rss.Channel.ManagingEditor);
			}
		}

		/// <summary>
		/// Writes the &lt;pubDate&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WritePubDate()
		{
			// <pubDate>Sun, 14 Oct 2007 05:00:00 GMT</pubDate>
			if (_rss.Channel.PublishedDate != null)
			{
				_writer.WriteElementString("pubDate", _rss.Channel.PublishedDate.Value.ToString("R"));
			}
		}

		/// <summary>
		/// Writes the &lt;rating&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteRating()
		{
			// <rating>(PICS-1.1 "http://www.rsac.org/ratingsv01.html" l by "webmaster@example.com" on "2007.01.29T10:09-0800" r (n 0 s 0 v 0 l 0))</rating>
			if (_rss.Channel.Rating != null)
			{
				_writer.WriteElementString("rating", _rss.Channel.Rating.ToString());
			}
		}

		/// <summary>
		/// Writes the &lt;ttl&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteTtl()
		{
			// <ttl>60</ttl>
			if (_rss.Channel.TimeToLive != null)
			{
				_writer.WriteElementString("ttl", _rss.Channel.TimeToLive.Value.ToString());
			}
		}

		/// <summary>
		/// Writes the &lt;webMaster&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteWebMaster()
		{
			// <webMaster>helpdesk@dallas.example.com</webMaster>
			if (!String.IsNullOrEmpty(_rss.Channel.WebMaster))
			{
				_writer.WriteElementString("webMaster", _rss.Channel.WebMaster);
			}
		}
		
		/// <summary>
		/// Writes the &lt;skipDays&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteSkipDays()
		{
			// <skipDays>
			//	 <day>Saturday</day>
			//	 <day>Sunday</day>
			// </skipDays>
			if (_rss.Channel.SkipDays.Count > 0)
			{
				_writer.WriteStartElement("skipDays");

				foreach (var day in _rss.Channel.SkipDays)
				{
					_writer.WriteElementString("day", day.ToString());
				}

				_writer.WriteEndElement();
			}
		}

		/// <summary>
		/// Writes the &lt;skipHours&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteSkipHours()
		{
			// <skipHours>
			//	 <hour>0</hour>
			//	 <hour>1</hour>
			//	 <hour>2</hour>
			//	 <hour>22</hour>
			//	 <hour>23</hour>
			//	</skipHours>
			if (_rss.Channel.SkipHours.Count > 0)
			{
				_writer.WriteStartElement("skipHours");

				foreach (var hour in _rss.Channel.SkipHours)
				{
					_writer.WriteElementString("hour", hour.ToString());
				}

				_writer.WriteEndElement();
			}
		}

		/// <summary>
		/// Writes the &lt;item&gt; node of the channel in the RSS feed.
		/// </summary>
		protected void WriteItems()
		{
			// <item>
			//	 <guid isPermaLink="false">tag:dallas.example.com,4131:news</guid>
			//   <title>Seventh Heaven! Ryan Hurls Another No Hitter</title>
			//	 <author>jbb@dallas.example.com (Joe Bob Briggs)</author>
			//	 <category domain="rec.arts.movies.reviews">1983/V</category>
			//	 <comments>http://dallas.example.com/feedback/1983/06/joebob.htm</comments>
			//	 <description><![CDATA[Texas Rangers pitcher Nolan Ryan hurled]]></description>
			//	 <enclosure length="24986239" type="audio/mpeg" url="http://dallas.example.com/joebob_050689.mp3" />
			//	 <link>http://dallas.example.com/1991/05/02/nolan.htm</link>
			//	 <pubDate>Fri, 05 Oct 2007 09:00:00 CST</pubDate>
			//	 <source url="http://la.example.com/rss.xml">Los Angeles Herald-Examiner</source>
			// </item>
			foreach (var item in _rss.Channel.Items)
			{
				_writer.WriteStartElement("item");

				if (item.Id != null)
				{
					_writer.WriteStartElement("guid");

					if (item.Id.IsPermaLink)
					{
						_writer.WriteString(item.Id.PermaLink.ToString());
					}
					else
					{
						_writer.WriteAttributeString("isPermaLink", "false");
						_writer.WriteString(item.Id.Tag);
					}

					_writer.WriteEndElement();
				}

				_writer.WriteElementString("title", item.Title);

				if (!String.IsNullOrEmpty(item.Author))
				{
					_writer.WriteElementString("author", item.Author);
				}

				if (item.Category != null)
				{
					_writer.WriteStartElement("category");
					_writer.WriteString(item.Category.Name);

					if (!String.IsNullOrEmpty(item.Category.Domain))
					{
						_writer.WriteAttributeString("domain", item.Category.Domain);
					}

					_writer.WriteEndElement();
				}

				if (item.Comments != null)
				{
					_writer.WriteElementString("comments", item.Comments.ToString());
				}

				if (!String.IsNullOrEmpty(item.Description))
				{
					_writer.WriteStartElement("description");
					_writer.WriteCData(item.Description);
					_writer.WriteEndElement();
				}

				if (item.Enclosure != null)
				{
					_writer.WriteStartElement("enclosure");
					_writer.WriteAttributeString("length", item.Enclosure.Length.ToString());
					_writer.WriteAttributeString("type", item.Enclosure.MimeType);
					_writer.WriteAttributeString("url", item.Enclosure.Url.ToString());
					_writer.WriteEndElement();
				}

				if (item.Link != null)
				{
					_writer.WriteElementString("link", item.Link.ToString());
				}

				if (item.PublishedDate != null)
				{
					_writer.WriteElementString("pubDate", item.PublishedDate.Value.ToString("R"));
				}

				if (item.ItemSource != null)
				{
					_writer.WriteStartElement("source");
					_writer.WriteAttributeString("url", item.ItemSource.Url.ToString());
					_writer.WriteString(item.ItemSource.Title);
					_writer.WriteEndElement();
				}

				_writer.WriteEndElement();
			}
		}
	}
}
